<?php
/**
 * @file
 * Helper functions definition
 */


/**
 * Check if apache or the current php user can write to all
 * drupal files
 * @param $dir
 */
function _cms_updater_check_fileperms($dir) {
  $source = _cms_updater_sanitizePath($dir);
  $files = scandir($dir);

  static $retarray = array(
    'success'        => TRUE,
    'erronfiles'     => array(),
    'successonfiles' => array(),
  );

  $ecp = FALSE;
  foreach ($files as $fname) {
    if ($fname != '.' && $fname != '..' && $fname != 'profiles' && $fname != 'sites' && $fname != '.htaccess' && $fname != 'robots.txt' && $fname != '.gitignore' && $fname != '.git') {
      $ecp = is_writable($source . '/' . $fname);
      if ($ecp === FALSE) {
        // Try chmod:
        if (is_dir($source . '/' . $fname)) {
          $orig_perms = _cms_updater_getFilePermission($source . '/' . $fname);
          if (@chmod($source . '/' . $fname, octdec('775'))) {
            $ecp = is_writable($source . '/' . $fname);
            chmod($source . '/' . $fname, octdec($orig_perms));
          }
        }
        else {
          $orig_perms = _cms_updater_getFilePermission($source . '/' . $fname);
          if (@chmod($source . '/' . $fname, octdec('775'))) {
            $ecp = is_writable($source . '/' . $fname);
            chmod($source . '/' . $fname, octdec($orig_perms));
          }
        }
      }
      $fileownerid = fileowner($source . '/' . $fname);
      if (function_exists('posix_getpwuid')) {
        $fileownerinfo = posix_getpwuid($fileownerid);
      }
      else {
        // File owner of current script
        $fileownerinfo = array('name' => 'not determindable');
      }
      if ($ecp === FALSE) {
        $error_case_file_perms = _cms_updater_getFilePermission($source . '/' . $fname);
        $retarray['success'] = FALSE;
        $retarray['error_on_files'][] = array(
          'filename'    => $source . '/' . $fname,
          'perms'       => $error_case_file_perms,
          'fileowner'   => $fileownerinfo['name'],
        );
      }
      else {
        $success_case_file_perms = _cms_updater_getFilePermission($source . '/' . $fname);
        $retarray['success_on_files'][] = array(
          'filename'    => $source . '/' . $fname,
          'perms'       => $success_case_file_perms,
          'fileowner'   => $fileownerinfo['name'],
        );
      }
      if (is_dir($source . '/' . $fname)) {
        _cms_updater_check_fileperms($source . '/' . $fname);
      }
    }
  }

  return $retarray;
}

/**
 * Get file permissions
 *
 * @param $file
 * @return string - last 3 diggits i.e. 777, 755
 */
function _cms_updater_getFilePermission($file) {
  $length = strlen(decoct(fileperms($file))) - 3;
  return substr(decoct(fileperms($file)), $length);
}

/**
 * Sanitize path
 *
 * @param $path
 * @return mixed|string
 */
function _cms_updater_sanitizePath($path) {
  $path = str_replace('\\', '/', $path); // Windows path sanitization.
  if (substr($path, -1) == '/') {
    $path = substr($path, 0, -1);
  }
  return $path;
}

/**
 * check if a writable temp-dir is determinable. this is needed for extracting
 * the downloaded new CMS/Drupal core files to.
 *
 * @return bool
 */
function _cms_updater_check_get_writable_temp_dir() {
  $dir = file_directory_temp();
  if ((file_exists($dir)) && (is_writable($dir))) {
    return TRUE;
  }
  else {
    return FALSE;
  }
}

/**
 * List some files
 *
 * @param $array
 * @return string
 */
function _cms_updater_files_array_to_list($array) {
  static $output = '</br>';
  static $i = 1;
  foreach ($array as $key => $item) {
    if (is_array($item)) {
      _cms_updater_files_array_to_list($item);
    }
    else {
      if (($key == 'filename') && ($i <= 10)) {
        $output .= $item . '</br>';
        $i++;
      }
    }
  }
  return $output;
}

/**
 * Recursive add files from source to destination zip Archive
 *
 * @param $src
 * @param $dst
 * @param $exclude
 */
function _cms_updater_recurse_zip($src, $dst, $exclude = array(), $include = array(), ZipArchive $zip) {
  if (!file_exists($dst)) {
    $zip->addEmptyDir($dst);
  }
  $dir = opendir($src);
  while (FALSE !== ($file = readdir($dir))) {
    // Do only include provided files if set
    if (!empty($include) && !in_array($file, $include)) {
      continue;
    }
    // Exclude certain files
    if (!in_array($file, $exclude)) {
      if (($file != '.') && ($file != '..')) {
        if (is_dir($src . '/' . $file)) {
          $zip->addEmptyDir($dst . '/' . $file . '/');
          $ret = _cms_updater_recurse_zip($src . '/' . $file, $dst . '/' . $file, array(), array(), $zip);
          if (is_array($ret)) {
            return $ret;
          }
        }
        else {
          if (!$zip->addFile($src . '/' . $file, $dst . '/' . $file)) {
            return array(
              'success'     => FALSE,
              'error'       => 'could not add file to zip',
              'file'        => $file,
              'src'         => $src,
              'destination' => $dst . '/' . $file,
              'base'        => basename($dst),
              'strrplc'     => str_replace($src . '/', '', $file),
              'zipstatus'   => $zip->getStatusString(),
            );
          }
        }
      }
    }
  }
  closedir($dir);
  return TRUE;
}

/**
 * Recursive copy files from source to destination
 *
 * @param $src
 * @param $dst
 * @param $exclude
 * @param $include
 */
function _cms_updater_recurse_copy($src, $dst, $exclude = array(), $include = array()) {
  // Create dir if not exist
  if (!file_exists($dst)) {
    @mkdir($dst);

  }

  $dir = opendir($src);
  while (FALSE !== ($file = readdir($dir))) {

    // Do only include provided files if set
    if (!empty($include) && !in_array($file, $include)) {
      continue;
    }
    // Exclude certain files
    if (!in_array($file, $exclude)) {
      if (($file != '.') && ($file != '..')) {
        if (is_dir($src . '/' . $file)) {
          $ret = _cms_updater_recurse_copy($src . '/' . $file, $dst . '/' . $file);
          // If we receive an error array - stop and return
          if (is_array($ret)) {
            return $ret;
          }
        }
        else {
          $ecp = copy($src . '/' . $file, $dst . '/' . $file);
          if ($ecp === FALSE) {
            return array(
              'success'     => FALSE,
              'error'       => 'could not copy to backup',
              'file'        => $file,
              'destination' => $dst . '/' . $file,
            );
          }
        }
      }
    }
  }
  closedir($dir);
  return TRUE;
}
